# YOLOv5 🚀 by Ultralytics, GPL-3.0 license
"""
Download utils
"""

import os
import wget
import platform
import subprocess
import time
import urllib
from pathlib import Path
from zipfile import ZipFile

import requests
import torch


def gsutil_getsize(url=''):
    # gs://bucket/file size https://cloud.google.com/storage/docs/gsutil/commands/du
    s = subprocess.check_output(f'gsutil du {url}', shell=True).decode('utf-8')
    return eval(s.split(' ')[0]) if len(s) else 0  # bytes


def safe_download(file, url, url2=None, min_bytes=1E0, error_msg=''):
    # Attempts to download file from url or url2, checks and removes incomplete downloads < min_bytes
    file = Path(file)
    assert_msg = f"Downloaded file '{file}' does not exist or size is < min_bytes={min_bytes}"
    try:  # url1
        LOGGER.info(f'Downloading {url} to {file}...')
        torch.hub.download_url_to_file(url, str(file), progress=False)
        assert file.exists() and file.stat().st_size > min_bytes, assert_msg  # check
    except Exception as e:  # url2
        file.unlink(missing_ok=True)  # remove partial downloads
        print(f'ERROR: {e}\nRe-attempting {url2 or url} to {file}...')
        wget.download(url, str(file))
    finally:
        if not file.exists() or file.stat().st_size < min_bytes:  # check
            file.unlink(missing_ok=True)  # remove partial downloads
            print(f"ERROR: {assert_msg}\n{error_msg}")
        print('')


def attempt_download(file, repo='ultralytics/yolov5'):  # from utils.downloads import *; attempt_download()
    # Attempt file download if does not exist
    file = Path(str(file).strip().replace("'", ''))

    if not file.exists():
        # URL specified
        name = Path(urllib.parse.unquote(str(file))).name  # decode '%2F' to '/' etc.
        if str(file).startswith(('http:/', 'https:/')):  # download
            url = str(file).replace(':/', '://')  # Pathlib turns :// -> :/
            file = name.split('?')[0]  # parse authentication https://url.com/file.txt?auth...
            if Path(file).is_file():
                print(f'Found {url} locally at {file}')  # file already exists
            else:
                safe_download(file=file, url=url, min_bytes=1E5)
            return file

        # GitHub assets
        file.parent.mkdir(parents=True, exist_ok=True)  # make parent dir (if required)
        try:
            response = requests.get(f'https://api.github.com/repos/{repo}/releases/latest').json()  # github api
            assets = [x['name'] for x in response['assets']]  # release assets, i.e. ['yolov5s.pt', 'yolov5m.pt', ...]
            tag = response['tag_name']  # i.e. 'v1.0'
        except:  # fallback plan
            assets = ['yolov5n.pt', 'yolov5s.pt', 'yolov5m.pt', 'yolov5l.pt', 'yolov5x.pt',
                      'yolov5n6.pt', 'yolov5s6.pt', 'yolov5m6.pt', 'yolov5l6.pt', 'yolov5x6.pt']
            try:
                tag = subprocess.check_output('git tag', shell=True, stderr=subprocess.STDOUT).decode().split()[-1]
            except:
                tag = 'v6.0'  # current release

        if name in assets:
            safe_download(file,
                          url=f'https://github.com/{repo}/releases/download/{tag}/{name}',
                          # url2=f'https://storage.googleapis.com/{repo}/ckpt/{name}',  # backup url (optional)
                          min_bytes=1E5,
                          error_msg=f'{file} missing, try downloading from https://github.com/{repo}/releases/')

    return str(file)


def gdrive_download(id='16TiPfZj7htmTyhntwcZyEEAejOUxuT6m', file='tmp.zip'):
    # Downloads a file from Google Drive. from yolov5.utils.downloads import *; gdrive_download()
    t = time.time()
    file = Path(file)
    cookie = Path('cookie')  # gdrive cookie
    print(f'Downloading https://drive.google.com/uc?export=download&id={id} as {file}... ', end='')
    file.unlink(missing_ok=True)  # remove existing file
    cookie.unlink(missing_ok=True)  # remove existing cookie

    # Attempt file download
    out = "NUL" if platform.system() == "Windows" else "/dev/null"
    os.system(f'curl -c ./cookie -s -L "drive.google.com/uc?export=download&id={id}" > {out}')
    if os.path.exists('cookie'):  # large file
        s = f'curl -Lb ./cookie "drive.google.com/uc?export=download&confirm={get_token()}&id={id}" -o {file}'
    else:  # small file
        s = f'curl -s -L -o {file} "drive.google.com/uc?export=download&id={id}"'
    r = os.system(s)  # execute, capture return
    cookie.unlink(missing_ok=True)  # remove existing cookie

    # Error check
    if r != 0:
        file.unlink(missing_ok=True)  # remove partial
        print('Download error ')  # raise Exception('Download error')
        return r

    # Unzip if archive
    if file.suffix == '.zip':
        print('unzipping... ', end='')
        ZipFile(file).extractall(path=file.parent)  # unzip
        file.unlink()  # remove zip

    print(f'Done ({time.time() - t:.1f}s)')
    return r


def get_token(cookie="./cookie"):
    with open(cookie) as f:
        for line in f:
            if "download" in line:
                return line.split()[-1]
    return ""

# Google utils: https://cloud.google.com/storage/docs/reference/libraries ----------------------------------------------
#
#
# def upload_blob(bucket_name, source_file_name, destination_blob_name):
#     # Uploads a file to a bucket
#     # https://cloud.google.com/storage/docs/uploading-objects#storage-upload-object-python
#
#     storage_client = storage.Client()
#     bucket = storage_client.get_bucket(bucket_name)
#     blob = bucket.blob(destination_blob_name)
#
#     blob.upload_from_filename(source_file_name)
#
#     print('File {} uploaded to {}.'.format(
#         source_file_name,
#         destination_blob_name))
#
#
# def download_blob(bucket_name, source_blob_name, destination_file_name):
#     # Uploads a blob from a bucket
#     storage_client = storage.Client()
#     bucket = storage_client.get_bucket(bucket_name)
#     blob = bucket.blob(source_blob_name)
#
#     blob.download_to_filename(destination_file_name)
#
#     print('Blob {} downloaded to {}.'.format(
#         source_blob_name,
#         destination_file_name))



import os
import re
import time
import logging
from .yolov5 import predict
from confusables import normalize
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC

logging.disable(logging.CRITICAL)
logging.basicConfig(level=logging.INFO)
log = logging.getLogger(__name__)
weights_path = os.path.join(os.path.dirname(__file__), "yolov5/yolov5x6.pt")

labels = [
    'bicycle', 'car', 'motorcycle', 'airplane', 'bus',
    'train', 'truck', 'boat', 'traffic light', 'fire hydrant'
]

def direct_prediction(image):
    return predict(weights=weights_path, image=image, only_labels=True)

class AISolver():
    label = ''
    timeout = 10
    driver = None
    
    def __init__(self, driver):
        self.driver = driver

    def start(self):
        self.type = 'h'
        self.solve_hcaptcha()
        
    def click_checkbox(self):
        iframe = WebDriverWait(self.driver, self.timeout).until(
            EC.presence_of_element_located(
                (By.CSS_SELECTOR, 'iframe[src *= "frame=checkbox"]' if self.type == 'h' else '.g-recaptcha iframe')
            )
        )

        log.info('Clicking on check box')
        self.driver.switch_to.frame(iframe)

        WebDriverWait(self.driver, self.timeout).until(
            EC.presence_of_element_located(
                (By.CSS_SELECTOR, '#checkbox' if self.type == 'h' else '.recaptcha-checkbox')
            )
        ).click()
        self.driver.switch_to.default_content()
        log.info('Checkbox clicked')

    def get_label(self):
        while True:
            raw_label = WebDriverWait(self.driver, self.timeout).until(
                EC.presence_of_element_located(
                    (By.CSS_SELECTOR, '.prompt-text' if self.type == 'h' else '.rc-imageselect-instructions')
                )
            ).text

            raw_label = normalize(raw_label, prioritize_alpha=True)[0].lower()

            for label in labels:
                if label in raw_label:
                    self.label = label
                    break
            
            # If the label is not in available classes, reload
            if self.label:
                print('Challenge label: ' + self.label)
                break
            else:
                WebDriverWait(self.driver, self.timeout).until(
                    EC.presence_of_element_located(
                        (By.CSS_SELECTOR, '.refresh.button' if self.type == 'h' else '#recaptcha-reload-button')
                    )
                ).click()
                time.sleep(2)

    def solve_hcaptcha(self):
        self.label = ""

        self.click_checkbox()

        # Get challenge label
        log.info('Getting the challenge\'s label!')
        iframe = WebDriverWait(self.driver, self.timeout).until(
            EC.presence_of_element_located((By.CSS_SELECTOR, 'iframe[src *= "frame=challenge"]'))
        )
        self.driver.switch_to.frame(iframe)

        self.get_label()

        # Get Images
        get_src = lambda i: i.value_of_css_property('background-image')[5:-2]

        log.info('Loading images...')
        while True:
            images = WebDriverWait(self.driver, self.timeout).until(
                EC.presence_of_all_elements_located((By.CSS_SELECTOR, '.task-image .image'))
            )
            
            loaded = all([
                get_src(image) for image in images
            ])
            
            if not loaded:
                continue

            for index, image in enumerate(images):
                src = get_src(image)
                result = direct_prediction(src)

                if self.label in result:
                    self.driver.execute_script(f"document.querySelectorAll('.task-image .image')[{index}].click()")
                    print(f'[*] Image {index+1} Result: {str(result)} | Status: True')
                else:
                    print(f'[*] Image {index+1} Result: {str(result)} | Status: False')

            self.driver.find_element(By.CSS_SELECTOR, '.button-submit.button').click()

            time.sleep(4)

            try:
                self.get_label()
            except:
                pass
            
            try:
                if not self.driver.find_element(By.CSS_SELECTOR, '.task-image .image'):
                    self.driver.switch_to.default_content()
                    break
            except:
                self.driver.switch_to.default_content()
                break


# YOLOv5 🚀 by Ultralytics, GPL-3.0 license

import argparse
import os
import sys
from pathlib import Path

import cv2
import torch
import torch.backends.cudnn as cudnn

model = None
FILE = Path(__file__).resolve()
ROOT = FILE.parents[0]  # YOLOv5 root directory
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH
ROOT = Path(os.path.relpath(ROOT, Path.cwd()))  # relative

from models.common import DetectMultiBackend
from utils.datasets import IMG_FORMATS, VID_FORMATS, LoadImages, LoadStreams
from utils.general import (LOGGER, check_file, check_img_size, check_imshow, colorstr,
                           increment_path, non_max_suppression, print_args, scale_coords, strip_optimizer, xyxy2xywh)
from utils.plots import Annotator, colors, save_one_box
from utils.torch_utils import select_device, time_sync


@torch.no_grad()
def predict(weights=ROOT / 'yolov5s.pt',  # model.pt path(s)
        image=ROOT / 'data/images',  # file/dir/URL/glob, 0 for webcam
        data=ROOT / 'models/coco128.yaml',  # dataset.yaml path
        imgsz=(640, 640),  # inference size (height, width)
        conf_thres=0.25,  # confidence threshold
        iou_thres=0.45,  # NMS IOU threshold
        max_det=1000,  # maximum detections per image
        device='',  # cuda device, i.e. 0 or 0,1,2,3 or cpu
        view_img=False,  # show results
        save_txt=False,  # save results to *.txt
        save_conf=False,  # save confidences in --save-txt labels
        save_crop=False,  # save cropped prediction boxes
        nosave=False,  # do not save images/videos
        classes=None,  # filter by class: --class 0, or --class 0 2 3
        agnostic_nms=False,  # class-agnostic NMS
        augment=False,  # augmented inference
        visualize=False,  # visualize features
        update=False,  # update all models
        project=ROOT / 'runs/detect',  # save results to project/name
        name='exp',  # save results to project/name
        exist_ok=False,  # existing project/name ok, do not increment
        line_thickness=3,  # bounding box thickness (pixels)
        hide_labels=False,  # hide labels
        hide_conf=False,  # hide confidences
        half=False,  # use FP16 half-precision inference
        dnn=False,  # use OpenCV DNN for ONNX inference
        only_labels=False, # only return the labels
        show_results=False, # show predicted result
        ):
    global model
    labels = []
    image = str(image)
    save_img = not nosave and not image.endswith('.txt')  # save inference images
    is_file = Path(image).suffix[1:] in (IMG_FORMATS + VID_FORMATS)
    is_url = image.lower().startswith(('rtsp://', 'rtmp://', 'http://', 'https://'))
    # webcam = image.isnumeric() or image.endswith('.txt') or (is_url)
    webcam = False
    if is_url:
        image = check_file(image, add_suffix=True if not is_file else False)  # download

    # Directories
    # save_dir = increment_path(Path(project) / name, exist_ok=exist_ok)  # increment run
    # (save_dir / 'labels' if save_txt else save_dir).mkdir(parents=True, exist_ok=True)  # make dir

    # Load model
    device = select_device(device)
    if not model:
        model = DetectMultiBackend(weights, device=device, dnn=dnn, data=data)
    stride, names, pt, jit, onnx, engine = model.stride, model.names, model.pt, model.jit, model.onnx, model.engine
    imgsz = check_img_size(imgsz, s=stride)  # check image size

    # Half
    half &= (pt or jit or onnx or engine) and device.type != 'cpu'  # FP16 supported on limited backends with CUDA
    if pt or jit:
        model.model.half() if half else model.model.float()

    # Dataloader
    if webcam:
        view_img = check_imshow()
        cudnn.benchmark = True  # set True to speed up constant image size inference
        dataset = LoadStreams(image, img_size=imgsz, stride=stride, auto=pt)
        bs = len(dataset)  # batch_size
    else:
        dataset = LoadImages(image, img_size=imgsz, stride=stride, auto=pt)
        bs = 1  # batch_size
    vid_path, vid_writer = [None] * bs, [None] * bs

    # Run inference
    model.warmup(imgsz=(1, 3, *imgsz), half=half)  # warmup
    dt, seen = [0.0, 0.0, 0.0], 0
    for path, im, im0s, vid_cap, s in dataset:
        t1 = time_sync()
        im = torch.from_numpy(im).to(device)
        im = im.half() if half else im.float()  # uint8 to fp16/32
        im /= 255  # 0 - 255 to 0.0 - 1.0
        if len(im.shape) == 3:
            im = im[None]  # expand for batch dim
        t2 = time_sync()
        dt[0] += t2 - t1

        # Inference
        visualize = increment_path(save_dir / Path(path).stem, mkdir=True) if visualize else False
        pred = model(im, augment=augment, visualize=visualize)
        t3 = time_sync()
        dt[1] += t3 - t2

        # NMS
        pred = non_max_suppression(pred, conf_thres, iou_thres, classes, agnostic_nms, max_det=max_det)
        dt[2] += time_sync() - t3

        # Second-stage classifier (optional)
        # pred = utils.general.apply_classifier(pred, classifier_model, im, im0s)

        # Process predictions
        for i, det in enumerate(pred):  # per image
            seen += 1
            if webcam:  # batch_size >= 1
                p, im0, frame = path[i], im0s[i].copy(), dataset.count
                s += f'{i}: '
            else:
                p, im0, frame = path, im0s.copy(), getattr(dataset, 'frame', 0)

            p = Path(p)  # to Path
            # save_path = str(save_dir / p.name)  # im.jpg
            # txt_path = str(save_dir / 'labels' / p.stem) + ('' if dataset.mode == 'image' else f'_{frame}')  # im.txt
            s += '%gx%g ' % im.shape[2:]  # print string
            gn = torch.tensor(im0.shape)[[1, 0, 1, 0]]  # normalization gain whwh
            imc = im0.copy() if save_crop else im0  # for save_crop
            annotator = Annotator(im0, line_width=line_thickness, example=str(names))
            if len(det):
                # Rescale boxes from img_size to im0 size
                det[:, :4] = scale_coords(im.shape[2:], det[:, :4], im0.shape).round()

                # Print results
                for c in det[:, -1].unique():
                    n = (det[:, -1] == c).sum()  # detections per class
                    s += f"{n} {names[int(c)]}{'s' * (n > 1)}, "  # add to string
                    labels.append(names[int(c)])

                # Write results
                for *xyxy, conf, cls in reversed(det):
                    if save_txt:  # Write to file
                        xywh = (xyxy2xywh(torch.tensor(xyxy).view(1, 4)) / gn).view(-1).tolist()  # normalized xywh
                        line = (cls, *xywh, conf) if save_conf else (cls, *xywh)  # label format
                        with open(txt_path + '.txt', 'a') as f:
                            f.write(('%g ' * len(line)).rstrip() % line + '\n')

                    if save_img or save_crop or view_img:  # Add bbox to image
                        c = int(cls)  # integer class
                        label = None if hide_labels else (names[c] if hide_conf else f'{names[c]} {conf:.2f}')
                        annotator.box_label(xyxy, label, color=colors(c, True))
                        if save_crop:
                            save_one_box(xyxy, imc, file=save_dir / 'crops' / names[c] / f'{p.stem}.jpg', BGR=True)

            # Print time (inference-only)
            LOGGER.info(f'{s}Done. ({t3 - t2:.3f}s)')

            # Stream results
            im0 = annotator.result()
            if show_results:
                cv2.imshow("Image", im0)
                cv2.waitKey(0)

    # Print results
    t = tuple(x / seen * 1E3 for x in dt)  # speeds per image
    LOGGER.info(f'Speed: %.1fms pre-process, %.1fms inference, %.1fms NMS per image at shape {(1, 3, *imgsz)}' % t)
    if update:
        strip_optimizer(weights)  # update model (to fix imageChangeWarning)

    os.unlink(image)

    if only_labels:
        return labels


import os
import time
import json
import gzip
import logging
from solver import AISolver
from seleniumwire.webdriver import ChromeOptions
from seleniumwire.undetected_chromedriver import Chrome

logging.getLogger("seleniumwire").setLevel(logging.CRITICAL)
logging.basicConfig(level=logging.INFO)
log = logging.getLogger(__name__)

key = ""
host = ""
driver = None

urls = {
    'api': "https://hcaptcha.com/1/api.js",
    'config': "https://hcaptcha.com/checksiteconfig",
    'getcaptcha': "https://hcaptcha.com/getcaptcha",
    'checkcaptcha': "https://hcaptcha.com/checkcaptcha"
}

def request_interceptor(request):
    if urls['config'] in request.url:
        request.url = f"{urls['config']}?host={host}&sitekey=f5561ba9-8f1e-40ca-9b5b-a0b3f719ef34&sc=1&swa=1"
    elif urls['getcaptcha'] in request.url:
        modify = request.body.decode('utf-8').split("&")
        modify[2] = f"host={host}" 
        request.body = bytes("&".join(modify), 'utf-8')
        del request.headers['Content-Length']
        request.headers['Content-Length'] = str(len(request.body))
    elif urls['checkcaptcha'] in request.url and request.method == "POST":
        body = request.body.decode('utf-8')
        data = json.loads(body)
        data['serverdomain'] = host
        request.body = json.dumps(data).encode('utf-8')
        del request.headers['Content-Length']
        request.headers['Content-Length'] = str(len(request.body))

def response_interceptor(request, response):
    if urls['getcaptcha'] in request.url or urls['checkcaptcha'] in request.url:
        global key
        body = gzip.decompress(response.body).decode('utf-8')
        data = json.loads(body)
        try:
            if data["bypass-message"]:
                key = "" if urls['getcaptcha'] in request.url else False
        except:
            key = data['generated_pass_UUID']

def new(_host, sitekey, chromedriver_path="/bin/chromedriver", headless=True):
    global driver, host
    host = _host

    if not os.path.exists(chromedriver_path):
        raise Exception("Chromedriver not found!")

    options = ChromeOptions()
    options.add_argument("--headless") if headless else None
    driver = Chrome(executable_path=chromedriver_path, options=options)
    driver.request_interceptor = request_interceptor
    driver.response_interceptor = response_interceptor
    log.info("Initializing page...")

    page_content = f'''
        <script src="{urls['api']}" async defer></script>
        <div class="h-captcha" data-sitekey="{sitekey}"></div>
    '''
    driver.get(f"data:text/html;charset=utf-8,{page_content}")

    log.info("Waiting for remecap...")
    solver = AISolver(driver, _type="hcaptcha")
    solver.start()

    log.info("Waiting for the key!")
    while True:
        if key != "":
            driver.close()
            return key
        time.sleep(0.2)


import os
import setuptools

with open("/home/runner/termux/remecap/README.md", "r") as fh:
    long_description = fh.read()

with open('/home/runner/termux/remecap/requirements.txt') as f:
    required = f.read().splitlines()

setuptools.setup(
    name="remecap",
    version="1.5.0",
    author="jiroawesome",
    description="A simple python library that bypasses H-Captcha automatically using yolov5",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jiroawesome/remecap",
    project_urls={
        "Bug Tracker": "https://github.com/jiroawesome/remecap/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    package_data={'': ['**/*']},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.6",
    install_requires=required
)


import os
import hashlib
from .yolov5 import predict
from .utils.downloads import safe_download

weights_path = os.path.join(os.path.dirname(__file__), "yolov5x6.pt")
weights_hash = "6ce5c5bc16f21b0d9beef9c2e5d075728f3ed417fc1a76780e4b3e8aff667105"
weights_download_path = "https://github.com/MainSilent/Impulse/releases/download/1.0.0/yolov5x6.pt"

def sha256sum(filename):
    h  = hashlib.sha256()
    b  = bytearray(128*1024)
    mv = memoryview(b)
    with open(filename, 'rb', buffering=0) as f:
        for n in iter(lambda : f.readinto(mv), 0):
            h.update(mv[:n])
    return h.hexdigest()

def download():
    print("After download run the script again")

    safe_download(weights_path,
        url=weights_download_path,
        error_msg=f'\nPlease download the file from {weights_download_path} and move to {weights_path}')

if not os.path.exists(weights_path):
    print("Weights file does not exist")
    input("Press Enter to download: ")
    download()

if os.path.exists(weights_path) and sha256sum(weights_path) != weights_hash:
    print("Weights file is corrupted, The file hash does not match!")
    i = input("Download the file again?[y/N] ")

    if i.lower() == 'y':
        download()